//-----------------------------------------------------------------------------
// File: alphaBlend.CPP
//
// Desc: Apha blending example.
//
//-----------------------------------------------------------------------------

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include <windows.h>
#include <ddraw.h>
#include <stdio.h>
#include <stdarg.h>
#include <mmsystem.h> 
#include "ht_ddutil.h"



//-----------------------------------------------------------------------------
// Definitions
//-----------------------------------------------------------------------------
#define NAME                "Alpha Blending"
#define TITLE               "Alpha Blending"
#define SYSTEMMEMORY		1
#define VIDEOMEMORY			0


//-----------------------------------------------------------------------------
// Structures
//-----------------------------------------------------------------------------
typedef struct
{
    RGBQUAD depth;		// How many bits of precision per channel
    RGBQUAD Position;	// At what bit position does the color start
	UINT bpp;
} RGBFORMAT;


//-----------------------------------------------------------------------------
// Global data
//-----------------------------------------------------------------------------
LPDIRECTDRAW			g_pDD				= NULL;  // DirectDraw object
LPDIRECTDRAWSURFACE		g_pDDSPrimary		= NULL;  // DirectDraw primary surface
LPDIRECTDRAWSURFACE		g_pDDSBack			= NULL;  // DirectDraw back surface
LPDIRECTDRAWSURFACE		g_pDDSSprite		= NULL;  // Offscreen surface 1
LPDIRECTDRAWSURFACE		g_pDDSBackGrnd		= NULL;  // Offscreen surface 2
LPDIRECTDRAWSURFACE		g_pDDSWork			= NULL;  // Offscreen working surface
BOOL					g_bActive			= FALSE; // Is application active?
BOOL					g_isMMX				= FALSE;
RGBFORMAT				g_pixelFormat;
int						g_Blt_Type			= 1;
int						g_isTrans			= 0;
int						g_FrameCount		= 0;
int						g_StartFrameCount	= 0;
DWORD					g_StartTime			= timeGetTime();
DWORD					g_EndTime			= g_StartTime;
int						g_FrameRate			= 0;


//-----------------------------------------------------------------------------
// Function Prototypes
//-----------------------------------------------------------------------------
HRESULT TransAlphaFast(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
					   LONG lDestX, LONG lDestY, RECT srcRect, DWORD ColorKey, WORD BPP);

HRESULT TransAlpha(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
				   LONG lDestX, LONG lDestY, RECT srcRect, WORD ALPHA, DWORD ColorKey, WORD BPP);

HRESULT TransAlphaImproved(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
				   LONG lDestX, LONG lDestY, RECT srcRect, WORD ALPHA, DWORD ColorKey, WORD BPP);

HRESULT TransAlphaMMX(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest,
					RECT srcRect, RECT destRect, WORD ALPHA, DWORD ColorKey, WORD BPP);

BOOL __cdecl IsMMX(void);
BOOL GetRGBFormat ( LPDIRECTDRAWSURFACE surf, RGBFORMAT* rgb);


//-----------------------------------------------------------------------------
// Name: ReleaseAllObjects()
// Desc: Finished with all objects we use; release them
//-----------------------------------------------------------------------------
static void ReleaseAllObjects(void)
{
    if (g_pDD != NULL)
    {
        if (g_pDDSPrimary != NULL)
        {
            g_pDDSPrimary->Release();
            g_pDDSPrimary = NULL;
        }
        if (g_pDDSSprite != NULL)
        {
            g_pDDSSprite->Release();
            g_pDDSSprite = NULL;
        }
		if (g_pDDSBackGrnd != NULL)
        {
            g_pDDSBackGrnd->Release();
            g_pDDSBackGrnd = NULL;
        }
		if (g_pDDSWork != NULL)
        {
            g_pDDSWork->Release();
            g_pDDSWork = NULL;
        }

        g_pDD->Release();
        g_pDD = NULL;
    }
}




//-----------------------------------------------------------------------------
// Name: InitFail()
// Desc: This function is called if an initialization function fails
//-----------------------------------------------------------------------------
HRESULT InitFail(HWND hWnd, HRESULT hRet, LPCTSTR szError,...)
{
    char		szBuff[128];
    va_list		vl;

    va_start(vl, szError);
    vsprintf(szBuff, szError, vl);
    ReleaseAllObjects();
    MessageBox(hWnd, szBuff, TITLE, MB_OK);
    DestroyWindow(hWnd);
    va_end(vl);
    return hRet;
}



//-----------------------------------------------------------------------------
// Name: RestoreAll()
// Desc: Restore all lost objects
//-----------------------------------------------------------------------------
HRESULT RestoreAll(void)
{
    HRESULT hRet;

    hRet = g_pDDSPrimary->Restore();
    if (hRet == DD_OK)
    {
        hRet = g_pDDSSprite->Restore();
        if (hRet == DD_OK)
        {
            DDReLoadBitmap(g_pDDSSprite, "htlogo.bmp");
        }
		hRet = g_pDDSBackGrnd->Restore();
        if (hRet == DD_OK)
        {
            DDReLoadBitmap(g_pDDSBackGrnd, "backgrnd.bmp");
        }
    }

    return hRet;
}




//-----------------------------------------------------------------------------
// Name: UpdateFrame()
// Desc: Decide what needs to be blitted next, wait for flip to complete,
//       then flip the buffers.
//-----------------------------------------------------------------------------
void 
UpdateFrame(void)
{
    static DWORD    lastTickCount[4] = {0, 0, 0, 0};
    static int      currentFrame[3]  = {0, 0, 0};
    DWORD           delay[4]         = {50, 78, 13, 93};
    int             xpos[3]          = {288, 190, 416};
    int             ypos[3]          = {128, 300, 256};
    RECT            rcRect, destRect;
    HRESULT         hRet;
	static int      ALPHA            = 0;  
	static int		inc				 = 2;
	char            str[80];
	HDC				hdc;
	char			pString[30];
 

    // Blit the background to the working surface
    rcRect.left   = 0;
    rcRect.top    = 0;
    rcRect.right  = 640;
    rcRect.bottom = 480;

    while (TRUE)
    {
		DDBLTFX ddbltfx;
		
		ddbltfx.dwSize = sizeof(ddbltfx);
		ddbltfx.dwFillColor = 0;

        hRet = g_pDDSWork->BltFast(0, 0, g_pDDSBackGrnd, &rcRect, DDBLTFAST_NOCOLORKEY);
        if (hRet == DD_OK)
            break;
        if (hRet == DDERR_SURFACELOST)
        {
            hRet = RestoreAll();
            if (hRet != DD_OK)
                return;
        }
        if (hRet != DDERR_WASSTILLDRAWING)
            return;
    }
    if (hRet != DD_OK)
        return;


	// Blit the sprite to the working surface
    rcRect.left     = 0;
    rcRect.top      = 0;
    rcRect.right    = 320;
    rcRect.bottom   = 240;

	destRect.left   = 160;
    destRect.top    = 100;
    destRect.right  = 480;
    destRect.bottom = 340;

    while (TRUE)
    {
		switch(g_Blt_Type)
		{
		case 1:
			hRet = g_pDDSWork->BltFast(160, 100, g_pDDSSprite, &rcRect, DDBLTFAST_SRCCOLORKEY);
			sprintf(str,"Using DirectX BltFast");
			break;
		case 2:
			hRet = TransAlphaFast(g_pDDSSprite, g_pDDSWork, 160, 100, rcRect, 65535 - g_isTrans, g_pixelFormat.bpp);
			sprintf(str,"Using The fast 50/50 function");
			break;
		case 3:
			hRet = TransAlpha(g_pDDSSprite, g_pDDSWork, 160, 100, rcRect, ALPHA, 65535 - g_isTrans, g_pixelFormat.bpp);
			sprintf(str,"Using the generic alpha blender");
			break;
		case 4:
			hRet = TransAlphaImproved(g_pDDSSprite, g_pDDSWork, 160, 100, rcRect, ALPHA, 65535 - g_isTrans, g_pixelFormat.bpp);
			sprintf(str,"Using the advanced alpha blender");
			break;
		case 5:
			hRet = TransAlphaMMX(g_pDDSSprite, g_pDDSWork,rcRect, destRect, ALPHA, 65535 - g_isTrans, g_pixelFormat.bpp);
			sprintf(str,"Using the MMX alpha blender");
			break;
		}

        if (hRet == DD_OK)
            break;
        if (hRet == DDERR_SURFACELOST)
        {
            hRet = RestoreAll();
            if (hRet != DD_OK)
                return;
        }
        if (hRet != DDERR_WASSTILLDRAWING)
            return;
    }


	// Draw some text on the Working surface
	sprintf(pString,"FPS : %d", g_FrameRate);

	g_pDDSWork->GetDC(&hdc);

	SetBkMode(hdc, TRANSPARENT);
	SetTextColor(hdc, 0x00FFFF);
	TextOut(hdc, 100, 20, pString, strlen(pString));
	TextOut(hdc, 100, 35, str, strlen(str));

	g_pDDSWork->ReleaseDC(hdc);


	// Blit the working surface to the back buffer
    rcRect.left   = 0;
    rcRect.top    = 0;
    rcRect.right  = 640;
    rcRect.bottom = 480;

	while (TRUE)
    {
        hRet = g_pDDSBack->BltFast(0, 0, g_pDDSWork,
                                   &rcRect, DDBLTFAST_NOCOLORKEY);
        if (hRet == DD_OK)
            break;
        if (hRet == DDERR_SURFACELOST)
        {
            hRet = RestoreAll();
            if (hRet != DD_OK)
                return;
        }
        if (hRet != DDERR_WASSTILLDRAWING)
            return;
    }


    // Flip the surfaces
    while (TRUE)
    {
        hRet = g_pDDSPrimary->Flip(NULL, 0);
        if (hRet == DD_OK)
            break;
        if (hRet == DDERR_SURFACELOST)
        {
            hRet = RestoreAll();
            if (hRet != DD_OK)
                break;
        }
        if (hRet != DDERR_WASSTILLDRAWING)
            break;
    }

	// Increment ALPHA
	ALPHA += inc;
	if (ALPHA > 256)
	{
		ALPHA = 256;
		inc = -2;
	}
	else if (ALPHA < 0)
	{
		ALPHA = 0;
		inc = 2;
	}


}




//-----------------------------------------------------------------------------
// Name: WindowProc()
// Desc: The Main Window Procedure
//-----------------------------------------------------------------------------
long FAR PASCAL 
WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
        case WM_ACTIVATEAPP:
            // Pause if minimized or not the top window
            g_bActive = (wParam == WA_ACTIVE) || (wParam == WA_CLICKACTIVE);
            return 0L;

        case WM_DESTROY:
            // Clean up and close the app
            ReleaseAllObjects();
            PostQuitMessage(0);
            return 0L;

        case WM_KEYDOWN:
            // Handle any non-accelerated key commands
            switch (wParam)
            {
				case VK_F1:
					g_Blt_Type = 1;
					return 0L;
				case VK_F2:
					g_Blt_Type = 2;
					return 0L;
				case VK_F3:
					g_Blt_Type = 3;
					return 0L;
				case VK_F4:
					g_Blt_Type = 4;
					return 0L;
				case VK_F5:
					g_Blt_Type = 5;
					return 0L;
				case VK_F6:
					g_isTrans = 0;
					return 0L;
				case VK_F7:
					g_isTrans = 1;
					return 0L;
                case VK_ESCAPE:
                case VK_F12:
					PostMessage(hWnd, WM_CLOSE, 0, 0);
                    return 0L;
            }
            break;

        case WM_SETCURSOR:
            // Turn off the cursor since this is a full-screen app
            SetCursor(NULL);
            return TRUE;

    }
    return DefWindowProc(hWnd, message, wParam, lParam);
}




//-----------------------------------------------------------------------------
// Name: InitApp()
// Desc: Do work required for every instance of the application:
//          Create the window, initialize data
//-----------------------------------------------------------------------------
static HRESULT
InitApp(HINSTANCE hInstance, int nCmdShow)
{
    HWND                        hWnd;
    WNDCLASS                    wc;
    DDSURFACEDESC               ddsd;
    DDSCAPS                     ddscaps;
    HRESULT                     hRet;
    LPDIRECTDRAW                pDD;

    // Set up and register window class
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH )GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass(&wc);

    // Create a window
    hWnd = CreateWindowEx(WS_EX_TOPMOST,
                          NAME,
                          TITLE,
                          WS_POPUP,
                          0,
                          0,
                          GetSystemMetrics(SM_CXSCREEN),
                          GetSystemMetrics(SM_CYSCREEN),
                          NULL,
                          NULL,
                          hInstance,
                          NULL);

    if (!hWnd)
        return FALSE;
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);
    SetFocus(hWnd);

    ///////////////////////////////////////////////////////////////////////////
    // Create the main DirectDraw object
    ///////////////////////////////////////////////////////////////////////////
    hRet = DirectDrawCreate(NULL, &pDD, NULL);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "DirectDrawCreate FAILED");


    // Fetch DirectDraw4 interface
    hRet = pDD->QueryInterface(IID_IDirectDraw, (LPVOID *) & g_pDD);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "QueryInterface FAILED");


    // Get exclusive mode
    hRet = g_pDD->SetCooperativeLevel(hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "SetCooperativeLevel FAILED");


    // Set the video mode to 640x480x16
    hRet = g_pDD->SetDisplayMode(640, 480, 16);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "SetDisplayMode FAILED");


    // Create the primary surface with 1 back buffer
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
                          DDSCAPS_FLIP |
                          DDSCAPS_COMPLEX;
    ddsd.dwBackBufferCount = 1;
    hRet = g_pDD->CreateSurface(&ddsd, &g_pDDSPrimary, NULL);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "CreateSurface FAILED");


    // Get a pointer to the back buffer
    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
    hRet = g_pDDSPrimary->GetAttachedSurface(&ddscaps, &g_pDDSBack);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "GetAttachedSurface FAILED");


	// Create the offscreen working surface
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize         = sizeof(ddsd);
    ddsd.dwFlags        = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_SYSTEMMEMORY;
	ddsd.dwWidth        = 640;
	ddsd.dwHeight       = 480;
 
    hRet = g_pDD->CreateSurface(&ddsd, &g_pDDSWork, NULL);
    if (hRet != DD_OK)
        return InitFail(hWnd, hRet, "CreateSurface FAILED");


    // Create the offscreen surface, by loading our bitmap.
    g_pDDSSprite = DDLoadBitmap(g_pDD, "ht_logo.bmp", 0, 0, SYSTEMMEMORY);
    if (g_pDDSSprite == NULL)
        return InitFail(hWnd, hRet, "DDLoadBitmap FAILED");


	// Create the offscreen surface, by loading our bitmap.
    g_pDDSBackGrnd = DDLoadBitmap(g_pDD, "ht_bkgrnd.bmp", 0, 0, SYSTEMMEMORY);
    if (g_pDDSBackGrnd == NULL)
        return InitFail(hWnd, hRet, "DDLoadBitmap FAILED");


    // Set the color key for this bitmap (black)
    DDSetColorKey(g_pDDSSprite, RGB(255, 255, 255));


    // Determine if this machine has MMX
	g_isMMX = IsMMX();


	// Determine the pixel format
	if ( ! GetRGBFormat( g_pDDSBackGrnd, &g_pixelFormat ) )
		return DDERR_GENERIC;


    return DD_OK;
}




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Initialization, message loop
//-----------------------------------------------------------------------------
int PASCAL
WinMain(HINSTANCE hInstance,
        HINSTANCE hPrevInstance,
        LPSTR lpCmdLine,
        int nCmdShow)
{
    MSG                         msg;

    if (InitApp(hInstance, nCmdShow) != DD_OK)
        return FALSE;

    while (TRUE)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (!GetMessage(&msg, NULL, 0, 0))
                return msg.wParam;
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else if (g_bActive)
        {
            UpdateFrame();

			g_EndTime = timeGetTime();

			if (g_EndTime - g_StartTime > 1000)
			{
				g_FrameRate = (g_FrameCount - g_StartFrameCount) * 1000 / (g_EndTime - g_StartTime);
				g_StartTime = g_EndTime;
				g_StartFrameCount = g_FrameCount;
			}
			g_FrameCount++;
        }
        else
        {
            // Make sure we go to sleep if we have nothing else to do
            WaitMessage();
        }
    }
}


////////////////////////////////////////////////////////////////////////
// TransAlphaFast - 
////////////////////////////////////////////////////////////////////////

HRESULT TransAlphaFast(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
					   LONG lDestX, LONG lDestY, RECT srcRect, DWORD ColorKey, WORD BPP)
{
    int register i,j;
	int height,width;
    BYTE* lpSprite;
    BYTE* lpDest;
    LONG dPitch, SpritePitch;
    DWORD sColorKey;
    DWORD sTemp,dTemp;
    WORD sbuf,dbuf;
    DWORD Result;
	BOOL oddWidth = FALSE;
	DDSURFACEDESC srcDDSD, destDDSD;

    // Set height and width of SPRITE
    height = srcRect.bottom - srcRect.top;
    width  = srcRect.right - srcRect.left;

    // Lock down both surfaces for read and write
	ZeroMemory(&srcDDSD, sizeof(srcDDSD));
	srcDDSD.dwSize = sizeof(srcDDSD);
	src->Lock(NULL, &srcDDSD, DDLOCK_WAIT, NULL);
    
	ZeroMemory(&destDDSD, sizeof(destDDSD));
	destDDSD.dwSize = sizeof(destDDSD);
	dest->Lock(NULL, &destDDSD, DDLOCK_WAIT, NULL);

    // Set the pitch for both surfaces
    SpritePitch = srcDDSD.lPitch;
    dPitch      = destDDSD.lPitch;

    // Initialize the pointers to the upper left hand corner of the surface
    lpSprite = (BYTE*)srcDDSD.lpSurface;
    lpDest   = (BYTE*)destDDSD.lpSurface;

	// Get the color key for sprite surface
	sColorKey = ColorKey;

	// Start RGB Format switch statement
	switch(BPP)
	{
	case 8:
		// IMHO paletized modes are a thing of the past please feel free to 
		// implement this if you so desire.
		break;

	case 15:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card that uses the 5-5-5 format 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.


		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != sColorKey)
				{
					dTemp = *((WORD*)lpDest);
					*((WORD*)lpDest) = (WORD)(((sTemp & 0x7BDE) >> 1) + ((dTemp & 0x7BDE) >> 1));
				}

				lpDest   += 2;
				lpSprite += 2;
			}

			j = width;
			do
			{
				sTemp  = *((DWORD*)lpSprite);
				dTemp  = *((DWORD*)lpDest);
				Result = dTemp;

				if ((sTemp & 0xFFFF) != sColorKey)
				{
					Result &= 0xFFFF0000;
					Result |= ((sTemp & 0x7BDE) >> 1) + ((dTemp & 0x7BDE) >> 1);
				}
				if ((sTemp >> 16) != sColorKey)
				{
					Result &= 0xFFFF;
					Result |= ((sTemp & 0x7BDE0000) >> 1) + ((dTemp & 0x7BDE0000) >> 1);
				}

				*((DWORD*)lpDest) = Result;
				lpDest   += 4;
				lpSprite += 4;
			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 16:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.

		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != sColorKey)
				{
					dTemp = *((WORD*)lpDest);
					*((WORD*)lpDest) = (WORD)(((sTemp & 0xF7DE) >> 1) + ((dTemp & 0xF7DE) >> 1));
				}

				lpDest   += 2;
				lpSprite += 2;
			}

			j = width;
			do
			{
				sTemp  = *((DWORD*)lpSprite);
				dTemp  = *((DWORD*)lpDest);
				Result = dTemp;

				if ((sTemp & 0xFFFF) != sColorKey)
				{
					Result &= 0xFFFF0000;
					Result |= ((sTemp & 0xF7DE) >> 1) + ((dTemp & 0xF7DE) >> 1);
				}
				if ((sTemp >> 16) != sColorKey)
				{
					Result &= 0xFFFF;
					Result |= ((sTemp & 0xF7DE0000) >> 1) + ((dTemp & 0xF7DE0000) >> 1);
				}

				*((DWORD*)lpDest) = Result;
				lpDest   += 4;
				lpSprite += 4;

			}while (--j > 0);
			lpDest += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;

	case 24:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card capable of 24bit rendering 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 3);
		lpDest   += (lDestY * dPitch) + (lDestX * 3);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (3 * width));
		dbuf = (WORD)(dPitch - (3 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp &= 0xFFFFFF) != sColorKey)
				{
					dTemp  = *((DWORD*)lpDest);
					Result = ((sTemp & 0xFEFEFE) >> 1) + ((dTemp & 0xFEFEFE) >> 1);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest++;
				}
				else
				{
					lpDest += 3;
				}

				lpSprite += 3;

			}while (--j > 0);
			lpDest += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;

	case 32:
		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 4);
		lpDest   += (lDestY * dPitch) + (lDestX * 4);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (4 * width));
		dbuf = (WORD)(dPitch - (4 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp &= 0xFFFFFF) != sColorKey)
				{
					dTemp  = *((DWORD*)lpDest);
					Result = ((sTemp & 0xFEFEFE) >> 1) + ((dTemp & 0xFEFEFE) >> 1);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest += 2;
				}
				else
				{
					lpDest += 4;
				}

				lpSprite += 4;

			}while (--j > 0);
			lpDest += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;
	} // End RGB Format switch statement


    src->Unlock(NULL);
    dest->Unlock(NULL);

	return DD_OK;
}


//////////////////////////////////////////////////////////////////////////////////
// TransAlpha - 
//////////////////////////////////////////////////////////////////////////////////
HRESULT TransAlpha(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
				   LONG lDestX, LONG lDestY, RECT srcRect, WORD ALPHA, DWORD ColorKey, WORD BPP)
{
	int register i,j;
	int height,width;
	BYTE* lpSprite;
	BYTE* lpDest;
	WORD dPitch, SpritePitch;
	DWORD sColorKey;
	DWORD sTemp,dTemp;
	DWORD sr,sg,sb,dr,dg,db;
	WORD sbuf,dbuf;
	DWORD Result;
	BOOL oddWidth = FALSE;
	DDSURFACEDESC srcDDSD, destDDSD;

	
	// Check the ALPHA value
	if (ALPHA < 0)
		ALPHA = 0;
	else if (ALPHA > 256)
		ALPHA = 256;

	// Set height and width of SPRITE
    height = srcRect.bottom - srcRect.top;
    width = srcRect.right - srcRect.left; 

	// Lock down both surfaces for read and write
	ZeroMemory(&srcDDSD, sizeof(srcDDSD));
	srcDDSD.dwSize = sizeof(srcDDSD);
	src->Lock(NULL, &srcDDSD, DDLOCK_WAIT, NULL);
    
	ZeroMemory(&destDDSD, sizeof(destDDSD));
	destDDSD.dwSize = sizeof(destDDSD);
	dest->Lock(NULL, &destDDSD, DDLOCK_WAIT, NULL);

    // Get the color key for sprite surface
    sColorKey = ColorKey;

	// Set the pitch for both surfaces
    SpritePitch = (WORD)srcDDSD.lPitch;
    dPitch      = (WORD)destDDSD.lPitch;

    // Initialize the pointers to the upper left hand corner of surface
    lpSprite = (BYTE*)srcDDSD.lpSurface;
    lpDest   = (BYTE*)destDDSD.lpSurface;


	switch(BPP)
	{
	case 8:
		// IMHO paletized modes are a thing of the past please feel free to 
		// implement this if you so desire.
		break;

	case 15:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card that uses the 5-5-5 format 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////
		
		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.

		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != sColorKey)
				{
					dTemp = *((WORD*)lpDest);
					sb = sTemp & 0x1f;
					db = dTemp & 0x1f;
					sg = (sTemp >> 5) & 0x1f;
					dg = (dTemp >> 5) & 0x1f;
					sr = (sTemp >> 10) & 0x1f;
					dr = (dTemp >> 10) & 0x1f;

					*((WORD*)lpDest) = (WORD)((ALPHA * (sb - db) >> 8) + db |
						((ALPHA * (sg - dg) >> 8) + dg) << 5 |
						((ALPHA * (sr - dr) >> 8) + dr) << 10);
				}

				lpDest   += 2;
				lpSprite += 2;
			}
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if (sTemp != ( (DWORD)sColorKey | ((DWORD)sColorKey << 16)) )
				{
					dTemp = *((DWORD*)lpDest);
					Result = dTemp;

					if ((sTemp & 0xffff) != sColorKey)
					{
						sb = sTemp & 0x1f;
						db = dTemp & 0x1f;
						sg = (sTemp >> 5) & 0x1f;
						dg = (dTemp >> 5) & 0x1f;
						sr = (sTemp >> 10) & 0x1f;
						dr = (dTemp >> 10) & 0x1f;

						Result = Result & 0xffff0000;
						Result |= (DWORD)((ALPHA * (sb - db) >> 8) + db |
							((ALPHA * (sg - dg) >> 8) + dg) << 5 |
							((ALPHA * (sr - dr) >> 8) + dr) << 10);
					}
					if (((sTemp >> 16) & 0xffff) != sColorKey)
					{
						sb = (sTemp >> 16) & 0x1f;
						db = (dTemp >> 16) & 0x1f;
						sg = (sTemp >> 21) & 0x1f;
						dg = (dTemp >> 21) & 0x1f;
						sr = (sTemp >> 26) & 0x1f;
						dr = (dTemp >> 26) & 0x1f;

						Result = Result & 0xffff;
						Result |= (DWORD)( (ALPHA * (sb - db) >> 8) + db |
							((ALPHA * (sg - dg) >> 8) + dg) << 5 |
							((ALPHA * (sr - dr) >> 8) + dr) << 10 ) << 16;
					}

					*((DWORD*)lpDest) = Result;
				}
				lpDest   += 4;
				lpSprite += 4;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 16:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.


		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != sColorKey)
				{
					dTemp = *((WORD*)lpDest);
					sb = sTemp & 0x1f;
					db = dTemp & 0x1f;
					sg = (sTemp >> 5) & 0x3f;
					dg = (dTemp >> 5) & 0x3f;
					sr = (sTemp >> 11) & 0x1f;
					dr = (dTemp >> 11) & 0x1f;

					*((WORD*)lpDest) = (WORD)((ALPHA * (sb - db) >> 8) + db |
						((ALPHA * (sg - dg) >> 8) + dg) << 5 |
						((ALPHA * (sr - dr) >> 8) + dr) << 11);
				}

				lpDest   += 2;
				lpSprite += 2;
			}
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if (sTemp != ( (DWORD)sColorKey | ((DWORD)sColorKey << 16)) )
				{
					dTemp = *((DWORD*)lpDest);
					Result = dTemp;

					if ((sTemp & 0xffff) != sColorKey)
					{
						sb = sTemp & 0x1f;
						db = dTemp & 0x1f;
						sg = (sTemp >> 5) & 0x3f;
						dg = (dTemp >> 5) & 0x3f;
						sr = (sTemp >> 11) & 0x1f;
						dr = (dTemp >> 11) & 0x1f;

						Result = Result & 0xffff0000;
						Result |= (DWORD)((ALPHA * (sb - db) >> 8) + db |
							((ALPHA * (sg - dg) >> 8) + dg) << 5 |
							((ALPHA * (sr - dr) >> 8) + dr) << 11);
					}
					if (((sTemp >> 16) & 0xffff) != sColorKey)
					{
						sb = (sTemp >> 16) & 0x1f;
						db = (dTemp >> 16) & 0x1f;
						sg = (sTemp >> 21) & 0x3f;
						dg = (dTemp >> 21) & 0x3f;
						sr = (sTemp >> 27) & 0x1f;
						dr = (dTemp >> 27) & 0x1f;

						Result = Result & 0xffff;
						Result |= (DWORD)( (ALPHA * (sb - db) >> 8) + db |
							((ALPHA * (sg - dg) >> 8) + dg) << 5 |
							((ALPHA * (sr - dr) >> 8) + dr) << 11 ) << 16;
					}

					*((DWORD*)lpDest) = Result;
				}
				lpDest   += 4;
				lpSprite += 4;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 24:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card capable of 24bit rendering 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 3);
		lpDest   += (lDestY * dPitch) + (lDestX * 3);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (3 * width));
		dbuf = (WORD)(dPitch - (3 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp & 0xFFFFFF) != sColorKey)
				{
					dTemp = *((DWORD*)lpDest);
					sb = sTemp & 0xFF;
					db = dTemp & 0xFF;
					sg = (sTemp >> 8) & 0xFF;
					dg = (dTemp >> 8) & 0xFF;
					sr = (sTemp >> 16) & 0xFF;
					dr = (dTemp >> 16) & 0xFF;

					Result = (DWORD)((ALPHA * (sb - db) >> 8) + db |
						((ALPHA * (sg - dg) >> 8) + dg) << 8 |
						((ALPHA * (sr - dr) >> 8) + dr) << 16);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest++;
				}
				else
				{
					lpDest += 3;
				}

				lpSprite += 3;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;

	case 32:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 4);
		lpDest   += (lDestY * dPitch) + (lDestX * 4);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (4 * width));
		dbuf = (WORD)(dPitch - (4 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp & 0xFFFFFF) != sColorKey)
				{
					dTemp = *((DWORD*)lpDest);
					sb = sTemp & 0xFF;
					db = dTemp & 0xFF;
					sg = (sTemp >> 8) & 0xFF;
					dg = (dTemp >> 8) & 0xFF;
					sr = (sTemp >> 16) & 0xFF;
					dr = (dTemp >> 16) & 0xFF;

					Result = (DWORD)((ALPHA * (sb - db) >> 8) + db |
						((ALPHA * (sg - dg) >> 8) + dg) << 8 |
						((ALPHA * (sr - dr) >> 8) + dr) << 16);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest += 2;
				}
				else
				{
					lpDest += 4;
				}

				lpSprite += 4;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;
	} // End RGB Format switch statement


	src->Unlock(NULL);
	dest->Unlock(NULL);

	return DD_OK;
}

//////////////////////////////////////////////////////////////////////////////////
// TransAlphaImproved - 
//////////////////////////////////////////////////////////////////////////////////
HRESULT TransAlphaImproved(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest, 
				   LONG lDestX, LONG lDestY, RECT srcRect, WORD ALPHA, DWORD ColorKey, WORD BPP)
{
	int register i,j;
	int height,width;
	BYTE* lpSprite;
	BYTE* lpDest;
	WORD dPitch, SpritePitch;
	DWORD sColorKey;
	DWORD sTemp,dTemp;
	DWORD sr,sg,sb,dr,dg,db;
	WORD sbuf,dbuf;
	DWORD Result;
	BOOL oddWidth = FALSE;
	DDSURFACEDESC srcDDSD, destDDSD;
	DWORD REDC,GREENC, BLUEC;
	DWORD PLUS64;
	DWORD ALPHABY4;
	DWORD doubleColorKey;


	// Check the ALPHA value
	if (ALPHA < 0)
		ALPHA = 0;
	else if (ALPHA > 256)
		ALPHA = 256;

	// Set height and width of SPRITE
    height = srcRect.bottom - srcRect.top;
    width = srcRect.right - srcRect.left; 

	// Lock down both surfaces for read and write
	ZeroMemory(&srcDDSD, sizeof(srcDDSD));
	srcDDSD.dwSize = sizeof(srcDDSD);
	src->Lock(NULL, &srcDDSD, DDLOCK_WAIT, NULL);
    
	ZeroMemory(&destDDSD, sizeof(destDDSD));
	destDDSD.dwSize = sizeof(destDDSD);
	dest->Lock(NULL, &destDDSD, DDLOCK_WAIT, NULL);

    // Get the color key for sprite surface
    sColorKey = ColorKey;

	// Set the pitch for both surfaces
    SpritePitch = (WORD)srcDDSD.lPitch;
    dPitch      = (WORD)destDDSD.lPitch;

    // Initialize the pointers to the upper left hand corner of surface
    lpSprite = (BYTE*)srcDDSD.lpSurface;
    lpDest   = (BYTE*)destDDSD.lpSurface;

	// Do some work outside of the loop
	PLUS64         = 64 | (64 << 16);
	ALPHABY4      = (ALPHA / 4) | ((ALPHA / 4) << 16);
	doubleColorKey = ColorKey | (ColorKey << 16);

	switch(BPP)
	{
	case 8:
		// IMHO paletized modes are a thing of the past please feel free to 
		// implement this if you so desire.
		break;

	case 15:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card that uses the 5-5-5 format 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////
		
		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.

		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != sColorKey)
				{
					dTemp = *((WORD*)lpDest);
					sb = sTemp & 0x1f;
					db = dTemp & 0x1f;
					sg = (sTemp >> 5) & 0x1f;
					dg = (dTemp >> 5) & 0x1f;
					sr = (sTemp >> 10) & 0x1f;
					dr = (dTemp >> 10) & 0x1f;

					*((WORD*)lpDest) = (WORD)((ALPHA * (db - sb) >> 8) + sb |
						((ALPHA * (dg - sg) >> 8) + sg) << 5 |
						((ALPHA * (dr - sr) >> 8) + sr) << 10);
				}

				lpDest += 2;
				lpSprite += 2;
			}
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ( sTemp != doubleColorKey )
				{
					dTemp = *((DWORD*)lpDest);

					sb = sTemp & 0x001F001F;
					db = dTemp & 0x001F001F;
					sg = (sTemp >> 5)  & 0x001F001F;
					dg = (dTemp >> 5)  & 0x001F001F;
					sr = (sTemp >> 10) & 0x001F001F;
					dr = (dTemp >> 10) & 0x001F001F;

					BLUEC  = ((((ALPHA * ((sb + PLUS64) - db)) >> 8) + db) - ALPHABY4) & 0x001F001F;
					GREENC = (((((ALPHA * ((sg + PLUS64) - dg)) >> 8) + dg) - ALPHABY4) & 0x001F001F) << 5;
					REDC   = (((((ALPHA * ((sr + PLUS64) - dr)) >> 8) + dr) - ALPHABY4) & 0x001F001F) << 10;

					Result = BLUEC | GREENC | REDC;

					if ( (sTemp >> 16) == ColorKey )
							Result = (Result & 0xFFFF) | (dTemp & 0xFFFF0000);
					else if ( (sTemp & 0xFFFF) == ColorKey )
							Result = (Result & 0xFFFF0000) | (dTemp & 0xFFFF);

					*((DWORD*)lpDest) = Result;
				}
				lpDest    += 4;
				lpSprite  += 4;

			}while (--j > 0);

			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 16:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 2);
		lpDest   += (lDestY * dPitch) + (lDestX * 2);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2 * width));
		dbuf = (WORD)(dPitch - (2 * width));

		// Is the Sprite width odd or even?
		if (width % 2 == 1)
		{
			oddWidth = TRUE;
			width = (width - 1) / 2; //div by 2, processing 2 pixels at a time.
		}
		else
			width = width / 2;  //div by 2, processing 2 pixels at a time.


		i = height;
		do
		{
			if (oddWidth)
			{
				sTemp = *((WORD*)lpSprite);

				if (sTemp != ColorKey)
				{
					dTemp = *((WORD*)lpDest);
					sb = sTemp & 0x1f;
					db = dTemp & 0x1f;
					sg = (sTemp >> 5) & 0x3f;
					dg = (dTemp >> 5) & 0x3f;
					sr = (sTemp >> 11) & 0x1f;
					dr = (dTemp >> 11) & 0x1f;

					*((WORD*)lpDest) = (WORD)((ALPHA * (sb - db) >> 8) + db |
						((ALPHA * (sg - dg) >> 8) + dg) << 5 |
						((ALPHA * (sr - dr) >> 8) + dr) << 11);
				}

				lpDest   += 2;
				lpSprite += 2;
			}
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ( sTemp != doubleColorKey )
				{
					dTemp = *((DWORD*)lpDest);

					sb = sTemp & 0x001F001F;
					db = dTemp & 0x001F001F;
					sg = (sTemp >> 5)  & 0x003F003F;
					dg = (dTemp >> 5)  & 0x003F003F;
					sr = (sTemp >> 11) & 0x001F001F;
					dr = (dTemp >> 11) & 0x001F001F;

					BLUEC  = ((((ALPHA * ((sb + PLUS64) - db)) >> 8) + db) - ALPHABY4) & 0x001F001F;
					GREENC = (((((ALPHA * ((sg + PLUS64) - dg)) >> 8) + dg) - ALPHABY4) & 0x003F003F) << 5;
					REDC   = (((((ALPHA * ((sr + PLUS64) - dr)) >> 8) + dr) - ALPHABY4) & 0x001F001F) << 11;

					Result = BLUEC | GREENC | REDC;

					if ( (sTemp >> 16) == ColorKey )
							Result = (Result & 0xFFFF) | (dTemp & 0xFFFF0000);
					else if ( (sTemp & 0xFFFF) == ColorKey )
							Result = (Result & 0xFFFF0000) | (dTemp & 0xFFFF);

					*((DWORD*)lpDest) = Result;
				}
				lpDest    += 4;
				lpSprite  += 4;

			}while (--j > 0);

			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 24:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card capable of 24bit rendering 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 3);
		lpDest   += (lDestY * dPitch) + (lDestX * 3);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (3 * width));
		dbuf = (WORD)(dPitch - (3 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp & 0xFFFFFF) != sColorKey)
				{
					dTemp = *((DWORD*)lpDest);
					sb = sTemp & 0xFF;
					db = dTemp & 0xFF;
					sg = (sTemp >> 8) & 0xFF;
					dg = (dTemp >> 8) & 0xFF;
					sr = (sTemp >> 16) & 0xFF;
					dr = (dTemp >> 16) & 0xFF;

					Result = (DWORD)((ALPHA * (db - sb) >> 8) + sb |
						((ALPHA * (dg - sg) >> 8) + sg) << 8 |
						((ALPHA * (dr - sr) >> 8) + sr) << 16);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest++;
				}
				else
				{
					lpDest += 3;
				}

				lpSprite += 3;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;

	case 32:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + (srcRect.left * 4);
		lpDest   += (lDestY * dPitch) + (lDestX * 4);

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (4 * width));
		dbuf = (WORD)(dPitch - (4 * width));

		i = height;
		do
		{
			j = width;
			do
			{
				sTemp = *((DWORD*)lpSprite);

				if ((sTemp & 0xFFFFFF) != sColorKey)
				{
					dTemp = *((DWORD*)lpDest);
					sb = sTemp & 0xFF;
					db = dTemp & 0xFF;
					sg = (sTemp >> 8) & 0xFF;
					dg = (dTemp >> 8) & 0xFF;
					sr = (sTemp >> 16) & 0xFF;
					dr = (dTemp >> 16) & 0xFF;

					Result = (DWORD)((ALPHA * (db - sb) >> 8) + sb |
						((ALPHA * (dg - sg) >> 8) + sg) << 8 |
						((ALPHA * (dr - sr) >> 8) + sr) << 16);

					*((WORD*)lpDest) = (WORD)(Result & 0xFFFF);
					lpDest += 2;
					*lpDest = (BYTE)(Result >> 16);
					lpDest += 2;
				}
				else
				{
					lpDest += 4;
				}

				lpSprite += 4;

			}while (--j > 0);
			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);
		break;
	} // End RGB Format switch statement


	src->Unlock(NULL);
	dest->Unlock(NULL);

	return DD_OK;
}


//////////////////////////////////////////////////////////////////////////////////
// CDXSprite TransAlphaMMX - 
//////////////////////////////////////////////////////////////////////////////////
HRESULT TransAlphaMMX(LPDIRECTDRAWSURFACE src, LPDIRECTDRAWSURFACE dest,
   RECT srcRect, RECT destRect, WORD ALPHA, DWORD ColorKey, WORD BPP)
{
	int register i,j;
	int height,width;
	BYTE* lpSprite;
	BYTE* lpDest;
	WORD dPitch, SpritePitch;
	WORD sbuf,dbuf;
	DDSURFACEDESC srcDDSD, destDDSD;

	__int64 ALPHA64, COLORKEY64, ALPHABY4;
	__int64 MASKRED, MASKGREEN, MASKBLUE;
	__int64 ADD64     = 0x0040004000400040;

	// Check if the global g_isMMX flag is set
	if ( ! g_isMMX )
		return DDERR_GENERIC;

	// Check the ALPHA value
	if (ALPHA < 0)
		ALPHA = 0;
	else if (ALPHA > 256)
		ALPHA = 256;

	// Set height and width of SPRITE
    height = srcRect.bottom - srcRect.top;
    width =  srcRect.right - srcRect.left;

	// Lock down both surfaces for read and write
	ZeroMemory(&srcDDSD, sizeof(srcDDSD));
	srcDDSD.dwSize = sizeof(srcDDSD);
	src->Lock(NULL, &srcDDSD, DDLOCK_WAIT, NULL);

	ZeroMemory(&destDDSD, sizeof(destDDSD));
	destDDSD.dwSize = sizeof(destDDSD);
	dest->Lock(NULL, &destDDSD, DDLOCK_WAIT, NULL);

    // Set the pitch for both surfaces
    SpritePitch = (WORD)srcDDSD.lPitch;
    dPitch      = (WORD)destDDSD.lPitch;

    // Initialize the pointers to the first pixel in the rectangle
    lpSprite = (BYTE*)srcDDSD.lpSurface;
    lpDest   = (BYTE*)destDDSD.lpSurface;


	switch(BPP)
	{
	case 8:
		// IMHO paletized modes are a thing of the past please feel free to 
		// implement this if you so desire.
		break;

	case 15:  ////////////////////////////////////////////////////////////////////////
		      //
		      //  WARNING!!!: I do not have a video card that uses the 5-5-5 format 
		      //              this barnch of code has not be tested.
		      //
		      ////////////////////////////////////////////////////////////////////////

	
		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + srcRect.left * 2;
		lpDest   += (destRect.top * dPitch) + destRect.left * 2;

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2*width));
		dbuf = (WORD)(dPitch - (2*width));

		// Setup the color channel masks
		MASKRED   = 0xF800F800F800F800;
		MASKGREEN = 0x07E007E007E007E0;
		MASKBLUE  = 0x001F001F001F001F;

		_asm
		{
			movd mm2,ALPHA    ; Copy ALPHA into mm2
			punpcklwd mm2,mm2 ; Unpack mm2 - 0000 0000 00aa 00aa
			punpckldq mm2,mm2 ; Unpack mm2 - 00aa 00aa 00aa 00aa
			movq ALPHA64,mm2  ; Save the result into ALPHA64

			psrlw mm2, 2      ; Divide each ALPHA value by 4
			movq ALPHABY4,mm2 ; Save the result to ALPHABY4

			movd mm4,ColorKey ; Copy ColorKey into mm4
			punpcklwd mm4,mm4 ; Unpack mm4 - 0000 0000 cccc cccc
			punpckldq mm4,mm4 ; Unpack mm4 - cccc cccc cccc cccc
			movq COLORKEY64,mm4 ; Save the result into COLORKEY64

		}

		i = height;
		do
		{
			j = width/4;
			// If you see two lines together this means that I am guessing that these two
			// instructions will pair in the Pentium U & V pipes.
			_asm
			{
				push edi ;Save off to restore later
				push esi ;Save off to restore later


				mov edi,lpDest				; Move the destination pointer into edi
				mov esi,lpSprite			; Move the source pointer into esi

			SPAN_RUN_555: movq mm7,[edi]	; Copy the 8 bytes pointed to by edi into mm7
				movq mm6,[esi]				; Copy the 8 bytes pointed to by esi into mm6

				movq mm2,ALPHA64			; Copy ALPHA64 into mm2
				movq mm0,mm7				; RED - Copy mm7 to mm0, destination pixels

				pand mm0,MASKRED			; RED - and mm0 with the red channel mask [0r00 0r00 0r00 0r00]
				movq mm1,mm6				; RED - Copy mm6 to mm1, source pixels

				pand mm1,MASKRED			; RED - and mm1 with red channel mask [0r00 0r00 0r00 0r00]
				psrlw mm0,11				; RED - shift each pixel to the right by 11 [000r 000r 000r 000r]

				movq mm5,mm7				; GRN - Copy destination pixels (mm7) to mm5
				psrlw mm1,11				; RED - shift each pixel to the right by 11 [000r 000r 000r 000r]


				paddw mm1, ADD64


				movq mm3,mm6				; GRN - Copy source pixels (mm6) to mm3
				psubsw mm1,mm0				; RED - Subtract the destination from the source

				pand mm5,MASKGREEN			; GRN - and mm5 with the green channel mask [00g0 00g0 00g0 00g0]
				pmullw mm1,mm2				; RED - multiple the subtraction result by the ALPHA value

				pand mm3,MASKGREEN			; GRN - and mm3 with the green channel mask [00g0 00g0 00g0 00g0]
				psrlw mm5,5					; GRN - shift each pixel to the right by 5 [000g 000g 000g 000g]

				psrlw mm3,5					; GRN - Shift green source to the right by 5
				nop							; Do nothing


				paddw mm3, ADD64


				psrlw mm1,8					; RED - Divide by 256 this is done to avoid floating point math
				psubsw mm3,mm5				; GRN - Subtract the green source and destination components

				pmullw mm3,mm2				; GRN - Multiple the result from the subtraction by the alpha value
				paddw mm1,mm0				; RED - Add the destination red component value


				psubw mm1, ALPHABY4


				psllw mm1,11				; RED - Shift the red component back into place, 11 to the left
				movq mm0,mm7				; BLU - Copy the destination pixels to mm0

				pand mm0, MASKBLUE			; BLU - 'and' mm0 with the blue channel mask [000b 000b 000b 000b]
				psrlw mm3,8					; GRN - Divide by 256 this is done to avoid floating point math

				paddw mm3,mm5				; GRN - Add the destination green component value
				movq mm4, mm6				; BLU - Copy the source into mm4


				psubw mm3, ALPHABY4


				pand mm4, MASKBLUE			; BLU - 'and' mm0 with the blue channel mask [000b 000b 000b 000b]
				psllw mm3,5					; GRN - Shift the green component back into place, 5 to the left


				paddw mm4, ADD64


				psubsw mm4,mm0				; BLU - Subtract the blue source and destination components
				por mm1,mm3					; GRN - 'or' the red and green components results back together

				pmullw mm4,mm2				; BLU - Multiple the result from the subtraction by the alpha value
				movq mm3,COLORKEY64			; Load COLORKEY64 into mm4

				psrlw mm4,8					; BLU - Divide by 256 this is done to avoid floating point math
				pcmpeqw mm3,mm6				; Compare colorKey to original source

				paddw mm4,mm0				; BLU - Add the destination blue component value
				movq mm5,mm3				; Copy mm3 to mm5


				psubw mm4, ALPHABY4


				por mm1,mm4					; BLU - 'or' the blue result to the red and green results
				pand mm5,mm7				; 'and' mm5 and the original destination pixels

				pandn mm3,mm1				; not's mm3 then performs and "AND" with mm1

				por mm3,mm5					; And mm3 and mm5

				movq [edi],mm3				; Copy the 4 alpha blended pixels to the destination

				add edi,8					; Add 8 bytes to the destination pointer
				mov eax,dword ptr [j]		; Load the value of j into eax

				add esi,8					; Add 8 bytes to the source pointer
				sub eax,1					; subtract 1 from eax (j--)

				mov dword ptr [lpDest], edi ; Save the new destination pointer value
				mov dword ptr [j],eax		; save the new value back to j

				mov dword ptr [lpSprite], esi ; Save the new source pointer value
				cmp eax,0					; If eax = 0 we have set the flag

				jg SPAN_RUN_555				; if flag is zero finish else loop back for more

				emms						; Clean up the MMX registers

				pop esi ; Restore esi
				pop edi ; Restore edi
			}

			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;

	case 16:

		// Initialize the pointers to the first pixel in the rectangle
		lpSprite += (srcRect.top * SpritePitch) + srcRect.left * 2;
		lpDest   += (destRect.top * dPitch) + destRect.left * 2;

		// Set the horizontal padding
		sbuf = (WORD)(SpritePitch - (2*width));
		dbuf = (WORD)(dPitch - (2*width));

		// Setup the color channel masks
		MASKRED   = 0xF800F800F800F800;
		MASKGREEN = 0x07E007E007E007E0;
		MASKBLUE  = 0x001F001F001F001F;

		_asm
		{
			movd mm2,ALPHA    ; Copy ALPHA into mm2
			punpcklwd mm2,mm2 ; Unpack mm2 - 0000 0000 00aa 00aa
			punpckldq mm2,mm2 ; Unpack mm2 - 00aa 00aa 00aa 00aa
			movq ALPHA64,mm2  ; Save the result into ALPHA64

			psrlw mm2, 2      ; Divide each ALPHA value by 4
			movq ALPHABY4,mm2 ; Save the result to ALPHABY4

			movd mm4,ColorKey ; Copy ColorKey into mm4
			punpcklwd mm4,mm4 ; Unpack mm4 - 0000 0000 cccc cccc
			punpckldq mm4,mm4 ; Unpack mm4 - cccc cccc cccc cccc
			movq COLORKEY64,mm4 ; Save the result into COLORKEY64

		}

		i = height;
		do
		{
			j = width/4;
			// If you see two lines together this means that I am guessing that these two
			// instructions will pair in the Pentium U & V pipes.
			_asm
			{
				push edi ;Save off to restore later
				push esi ;Save off to restore later


				mov edi,lpDest				; Move the destination pointer into edi
				mov esi,lpSprite			; Move the source pointer into esi

			SPAN_RUN_565: movq mm7,[edi]	; Copy the 8 bytes pointed to by edi into mm7
				movq mm6,[esi]				; Copy the 8 bytes pointed to by esi into mm6

				movq mm2,ALPHA64			; Copy ALPHA64 into mm2
				movq mm0,mm7				; RED - Copy mm7 to mm0, destination pixels

				pand mm0,MASKRED			; RED - and mm0 with the red channel mask [0r00 0r00 0r00 0r00]
				movq mm1,mm6				; RED - Copy mm6 to mm1, source pixels

				pand mm1,MASKRED			; RED - and mm1 with red channel mask [0r00 0r00 0r00 0r00]
				psrlw mm0,11				; RED - shift each pixel to the right by 11 [000r 000r 000r 000r]

				movq mm5,mm7				; GRN - Copy destination pixels (mm7) to mm5
				psrlw mm1,11				; RED - shift each pixel to the right by 11 [000r 000r 000r 000r]


				paddw mm1, ADD64


				movq mm3,mm6				; GRN - Copy source pixels (mm6) to mm3
				psubsw mm1,mm0				; RED - Subtract the destination from the source

				pand mm5,MASKGREEN			; GRN - and mm5 with the green channel mask [00g0 00g0 00g0 00g0]
				pmullw mm1,mm2				; RED - multiple the subtraction result by the ALPHA value

				pand mm3,MASKGREEN			; GRN - and mm3 with the green channel mask [00g0 00g0 00g0 00g0]
				psrlw mm5,5					; GRN - shift each pixel to the right by 5 [000g 000g 000g 000g]

				psrlw mm3,5					; GRN - Shift green source to the right by 5
				nop							; Do nothing


				paddw mm3, ADD64


				psrlw mm1,8					; RED - Divide by 256 this is done to avoid floating point math
				psubsw mm3,mm5				; GRN - Subtract the green source and destination components

				pmullw mm3,mm2				; GRN - Multiple the result from the subtraction by the alpha value
				paddw mm1,mm0				; RED - Add the destination red component value


				psubw mm1, ALPHABY4


				psllw mm1,11				; RED - Shift the red component back into place, 11 to the left
				movq mm0,mm7				; BLU - Copy the destination pixels to mm0

				pand mm0, MASKBLUE			; BLU - 'and' mm0 with the blue channel mask [000b 000b 000b 000b]
				psrlw mm3,8					; GRN - Divide by 256 this is done to avoid floating point math

				paddw mm3,mm5				; GRN - Add the destination green component value
				movq mm4, mm6				; BLU - Copy the source into mm4


				psubw mm3, ALPHABY4


				pand mm4, MASKBLUE			; BLU - 'and' mm0 with the blue channel mask [000b 000b 000b 000b]
				psllw mm3,5					; GRN - Shift the green component back into place, 5 to the left


				paddw mm4, ADD64


				psubsw mm4,mm0				; BLU - Subtract the blue source and destination components
				por mm1,mm3					; GRN - 'or' the red and green components results back together

				pmullw mm4,mm2				; BLU - Multiple the result from the subtraction by the alpha value
				movq mm3,COLORKEY64			; Load COLORKEY64 into mm4

				psrlw mm4,8					; BLU - Divide by 256 this is done to avoid floating point math
				pcmpeqw mm3,mm6				; Compare colorKey to original source

				paddw mm4,mm0				; BLU - Add the destination blue component value
				movq mm5,mm3				; Copy mm3 to mm5


				psubw mm4, ALPHABY4


				por mm1,mm4					; BLU - 'or' the blue result to the red and green results
				pand mm5,mm7				; 'and' mm5 and the original destination pixels

				pandn mm3,mm1				; not's mm3 then performs and "AND" with mm1

				por mm3,mm5					; And mm3 and mm5

				movq [edi],mm3				; Copy the 4 alpha blended pixels to the destination

				add edi,8					; Add 8 bytes to the destination pointer
				mov eax,dword ptr [j]		; Load the value of j into eax

				add esi,8					; Add 8 bytes to the source pointer
				sub eax,1					; subtract 1 from eax (j--)

				mov dword ptr [lpDest], edi ; Save the new destination pointer value
				mov dword ptr [j],eax		; save the new value back to j

				mov dword ptr [lpSprite], esi ; Save the new source pointer value
				cmp eax,0					; If eax = 0 we have set the flag

				jg SPAN_RUN_565				; if flag is zero finish else loop back for more

				emms						; Clean up the MMX registers

				pop esi ; Restore esi
				pop edi ; Restore edi
			}

			lpDest   += dbuf;
			lpSprite += sbuf;

		}while (--i > 0);

		break;


	case 24:

		// Not Implemented
		break;

	case 32:

	    // Not implemented
		break;

	} // End RGB Format switch statement

	src->Unlock(NULL);
	dest->Unlock(NULL);

	return DD_OK;
}


////////////////////////////////////////////////////////////////////////
//
//  GetRGBFormat
//
//  Fills out a RGBFORMAT structure.
//
////////////////////////////////////////////////////////////////////////

BOOL GetRGBFormat ( LPDIRECTDRAWSURFACE surf, RGBFORMAT* rgb)
{
    DDSURFACEDESC ddsd;
    BYTE shiftcount;

	//get a surface description
    ddsd.dwSize = sizeof( ddsd );
    ddsd.dwFlags = DDSD_PIXELFORMAT;

    if (surf->GetSurfaceDesc( &ddsd ) != DD_OK )
        return FALSE;

	// Check if the surface is 4 or 8 bpp
	if (ddsd.ddpfPixelFormat.dwRGBBitCount <= 8)
	{
		rgb->bpp = ddsd.ddpfPixelFormat.dwRGBBitCount;
		
		rgb->depth.rgbRed      = 0;
		rgb->Position.rgbRed   = 0;

		rgb->depth.rgbGreen    = 0;
		rgb->Position.rgbGreen = 0;

		rgb->depth.rgbBlue     = 0;
		rgb->Position.rgbBlue  = 0;
		
		return TRUE;
	}

	//Get the RED channel data for 15, 16, 24, or 32 bpp surface
    shiftcount = 0;
    while ( !(ddsd.ddpfPixelFormat.dwRBitMask & 1) && (shiftcount < 32))
    {
        ddsd.ddpfPixelFormat.dwRBitMask >>= 1;
        shiftcount++;
    }

	// depth is the highest possible value that this channel could
	// be set to. It is cast to a BYTE because 255 is the highest
	// value for all modes.
    rgb->depth.rgbRed = (BYTE) ddsd.ddpfPixelFormat.dwRBitMask;

	// Position represents where the color channel starts, bitwise, 
	// in the pixel. Another way of saying this that Position is 
	// eqaul to the number of zeros to the right of the channel's
	// bitmask.
    rgb->Position.rgbRed = shiftcount;

    //Get the GREEN channel data for 15, 16, 24, or 32 bpp surface
    shiftcount = 0;
    while ( !(ddsd.ddpfPixelFormat.dwGBitMask & 1) && (shiftcount < 32))
    {
        ddsd.ddpfPixelFormat.dwGBitMask >>= 1;
        shiftcount++;
    }
    rgb->depth.rgbGreen =(BYTE)ddsd.ddpfPixelFormat.dwGBitMask;
    rgb->Position.rgbGreen = shiftcount;

    //Get the BLUE channel data for 15, 16, 24, or 32 bpp surface
    shiftcount = 0;
    while ( !(ddsd.ddpfPixelFormat.dwBBitMask & 1) && (shiftcount < 32))
    {
        ddsd.ddpfPixelFormat.dwBBitMask >>= 1;
        shiftcount++;
    }
    rgb->depth.rgbBlue =(BYTE)ddsd.ddpfPixelFormat.dwBBitMask;
    rgb->Position.rgbBlue = shiftcount;

	//  Determine the bpp for the surface. (15, 16, 24, or 32)
	if (ddsd.ddpfPixelFormat.dwFlags & DDPF_RGB)
	{
		if (ddsd.ddpfPixelFormat.dwRGBBitCount == 16)
		{
			if ((rgb->depth.rgbBlue + rgb->depth.rgbGreen + rgb->depth.rgbRed) == 93)
				rgb->bpp = 15;
			else
				rgb->bpp = 16;
		}
		else if (ddsd.ddpfPixelFormat.dwRGBBitCount == 24)
			rgb->bpp = 24;
		else if (ddsd.ddpfPixelFormat.dwRGBBitCount == 32)
			rgb->bpp = 32;
		else
			rgb->bpp = 0;
	}
	else
	{
		rgb->bpp = 0;
		return FALSE;
	}

	return TRUE;
}


////////////////////////////////////////////////////////////////////////
//
//  IsMMX
//
//  Is the machine capable of MMX instructions
//
////////////////////////////////////////////////////////////////////////

BOOL __cdecl IsMMX(void)
{
    SYSTEM_INFO si;
    int nCPUFeatures=0;
    GetSystemInfo(&si);
    if (si.dwProcessorType != PROCESSOR_INTEL_386 && si.dwProcessorType != PROCESSOR_INTEL_486)
    {
        try
        {
            __asm
            {
                ; we must push/pop the registers << CPUID>>  writes to, as the
				; optimiser doesn't know about << CPUID>> , and so doesn't expect
				; these registers to change.
                push eax
                push ebx
                push ecx
                push edx

                ; << CPUID>> 
                ; eax=0,1,2 -> CPU info in eax,ebx,ecx,edx
                mov eax,1
                _emit 0x0f
                _emit 0xa2
                mov nCPUFeatures,edx

                pop edx
                pop ecx
                pop ebx
                pop eax
            }
        }
        catch(...) // just to be sure...
        {
			return false;
        }
    }
    return (nCPUFeatures & 0x00800000) != 0;
}
